import os
import shutil

# __all__ = ["copy_num_folders", "copy_folders","batch_file_overwrite"]

def copy_num_folders(ref_dir:str, src_dir:str, dest_dir:str, num_folders=None):
    folders = [folder for folder in os.listdir(src_dir) if os.path.isdir(os.path.join(ref_dir, folder))]

    count = 0
    if(num_folders==None):
        for folder in folders[:]:
            count = count+1
            src_folder = os.path.join(src_dir, folder)
            dest_folder = os.path.join(dest_dir, folder)

            try:
                shutil.copytree(src_folder, dest_folder)
                print(str(count),"Copied "+src_folder+" to "+dest_folder)
            except Exception as e:
                print("Failed to copy "+src_folder+" to "+dest_folder+":" +str(e))
    else:
        for folder in folders[:num_folders]:
            count = count+1
            src_folder = os.path.join(src_dir, folder)
            dest_folder = os.path.join(dest_dir, folder)

            try:
                shutil.copytree(src_folder, dest_folder)
                print(str(count),"Copied "+src_folder+" to "+dest_folder)
            except Exception as e:
                print("Failed to copy "+src_folder+" to "+dest_folder+":" +str(e))        

    return None



def copy_folders(ref_dir:str, src_dir:str, dest_dir:str):
    folders = [folder for folder in os.listdir(ref_dir) if os.path.isdir(os.path.join(ref_dir,folder))]

    count = 0
    for folder in folders:
        count = count + 1
        src_folder = os.path.join(src_dir, folder)
        dest_folder = os.path.join(dest_dir, folder)

        try:
            shutil.copytree(src_folder, dest_folder)
            print(str(count), "Copied "+src_folder+" to "+dest_folder)
        except Exception as e:
            print("Failed to copy "+src_folder+" to "+dest_folder+": "+str(e))
            
    return None



def batch_file_overwrite(src_dir: str, dest_dir:str, filename: str):
    dest_folders = [folder for folder in os.listdir(dest_dir) if os.path.isdir(os.path.join(dest_dir, folder))]
    src_folders = [folder for folder in os.listdir(src_dir) if os.path.isdir(os.path.join(src_dir, folder))]

    count = 0
    for folder in dest_folders:
        if folder in src_folders:
            file_to_copy_from_src = os.path.join(src_dir, folder, filename)
            file_to_copy_over_in_dest_folder = os.path.join(dest_dir, folder, filename)

            try:
                count = count + 1
                shutil.copy(file_to_copy_from_src, file_to_copy_over_in_dest_folder)
                print(str(count), "Updated "+file_to_copy_over_in_dest_folder+" to "+file_to_copy_from_src)
            except Exception as e:
                print("Failed to update "+file_to_copy_over_in_dest_folder+" to "+file_to_copy_from_src+": "+str(e))
    return None


from .folder_structure_view import(
    get_folder_overview,
    # __all__ as __all_folder_structure_view__
)

from .clone import(
    copy_num_folders,
    copy_folders,
    batch_file_overwrite,
    # __all__ as __all_clone__
)

# from .folder_structure_view import __all__ as __all_folder_structure_view__
# from .clone import __all__ as __all_clone_

# __all__  = __all_folder_structure_view__ + __all_clone_

# # stop polluting name space
# del __all_folder_structure_view__
# del __all_clone_



import pandas as pd
import os
import glob
from pathlib import Path

# __all__ = ["get_folder_overview"]

def get_folder_overview(path: str):
    sub_folder_names = []
    file_names = []
    file_sizes = []
    file_extenstions = []
    num_files_folder = []
    folder_paths = []
    is_folder = []
    depth_count = []

    files = glob.glob(pathname=path+"/**/*", recursive=True)
    for file in files:

        # get the relative path from the base path ("path") to the target
        # path ("file")
        p = Path(os.path.relpath(file, path))
       
        
        if os.path.isfile(file):
            # print(file)
            
            dir_path = os.path.dirname(file)
            folder_paths.append(file)

            dir_name = os.path.basename(dir_path)
            sub_folder_names.append(dir_name)

            file_name = os.path.basename(file)
            file_names.append(file_name)

            file_size = os.path.getsize(file)
            file_sizes.append(file_size)

            file_extension = os.path.splitext(file_name)[1]
            file_extenstions.append(file_extension)

            num_files_folder.append(0)
            
            is_folder.append(False)

            depth_count.append(len(p.parents)-1)
            
        else:
            dir_name = os.path.basename(file)
            sub_folder_names.append(dir_name)
            
            file_names.append("N/A")
            file_size = os.path.getsize(0)
            file_sizes.append(file_size)

            file_extenstions.append("N/A")

            f = os.listdir(file)
            num_files = len(f)
            num_files_folder.append(num_files)

            folder_paths.append(file)
            is_folder.append(True)

            depth_count.append(len(p.parents)-1)

        
    df = pd.DataFrame({
        'Path':folder_paths,
        'Folder Name':sub_folder_names,
        'Is Folder?': is_folder,
        'File Name':file_names,
        'File Size (Bytes)': file_sizes,
        'File Extensions': file_extenstions,
        'Number of Files in Folder': num_files_folder,
        'Depth': depth_count
        }   
    )

    return df


from setuptools import setup, find_packages
from pathlib import Path
import os

VERSION = "0.0.2"
# readme information
dir = Path(__file__).absolute().parent
with open(os.path.join(dir, "README.md")) as file: 
    README = file.read()

# required python packages
try: 
    dir = Path(__file__).absolute().parent
    with open(os.path.join(dir, "requirements.txt"), encoding='utf-8') as file: 
        packages = file.readlines()
    REQUIREMENTS = [package.strip() for package in packages]
except FileNotFoundError:
    REQUIREMENTS =[]


DESCRIPTION = "Sage is a python package that offers a detailed overview of folder contents and streamlines the process of copying and managing directories."
LONG_DESCRIPTION = README




# setting up
setup(
    name="sage_directory",
    version=VERSION,
    author="Maxine Attobrah",
    author_email="maxineattobrah@gmail.com",
    url = "https://github.com/maxineattobrah/Sage",
    description=DESCRIPTION,

    long_description=LONG_DESCRIPTION,
    packages = find_packages(),
    install_requires = REQUIREMENTS,
    long_description_content_type='text/markdown',
    keywords=["python","folder overview","folder management", "folder analysis","batch operations", "data science utility","data engineering utility", "IT operations"],
    classifiers=[
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'Intended Audience :: System Administrators',
    'Programming Language :: Python :: 3',
    'Operating System :: Microsoft :: Windows',
    'Operating System :: MacOS',
    'Operating System :: Unix',
    'Topic :: Scientific/Engineering',
    'Topic :: Scientific/Engineering :: Information Analysis',
    'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    platforms="any"

)